@@ -39,6 +39,7 @@ verbose(const char *fmt,...)
 
 	va_start(args, fmt);
 	vprintf(fmt, args);
+	fflush(stdout);
 	va_end(args);
 }
 
@@ -51,13 +52,12 @@ debug(const char *fmt,...)
 {
 	va_list args;
 
-	if (opts.debug <= 0 || !debugfp)
+	if (!opts.debug || !debugfp)
 		return;
 
 	va_start(args, fmt);
 	vfprintf(debugfp, fmt, args);
 	fflush(debugfp);
-
 	va_end(args);
 }
 
@@ -68,7 +68,7 @@ void
 debugc(char c)
 {
 
-	if (opts.debug <= 0 || !debugfp)
+	if (!opts.debug || !debugfp)
 		return;
 
 	fputc(c, debugfp);
@@ -86,6 +86,7 @@ error(const char *fmt,...)
 	va_start(args, fmt);
 	fprintf(stderr, "imapfilter: ");
 	vfprintf(stderr, fmt, args);
+	fflush(stderr);
 	va_end(args);
 	if (logfp) {
 		va_start(args, fmt);
@@ -110,6 +111,7 @@ fatal(unsigned int errnum, const char *fmt,...)
 	va_start(args, fmt);
 	fprintf(stderr, "imapfilter: ");
 	vfprintf(stderr, fmt, args);
+	fflush(stderr);
 	va_end(args);
 
 	if (logfp) {
