@@ -76,6 +76,7 @@ end
 
 function Mailbox._cached_close(self)
     if not self._check_connection(self) then return end
+    if self._account._account.selected == nil then return end
     local r = ifcore.close(self._account._account.session)
     self._check_result(self, 'close', r)
     if r == false then return false end
@@ -86,27 +87,39 @@ function Mailbox._cached_close(self)
 end
 
 
-function Mailbox._send_query(self, criteria, charset)
+function Mailbox._send_query(self, criteria, messages)
     _check_optional(criteria, { 'string', 'table' })
-    _check_optional(charset, 'string')
+    _check_optional(messages, 'table')
 
     if self._cached_select(self) ~= true then return {} end
 
-    local query
-    if criteria == nil then
-        query = 'ALL'
-    elseif type(criteria) == 'string' then
-        query = 'ALL ' .. criteria
-    else 
-        query = _make_query(criteria)
+    local mesgs
+    if messages == nil then
+        mesgs = nil
+    else
+        mesgs = _extract_messages(self, messages)
+    end
+    if mesgs == nil or #mesgs == 0 or
+       options.limit > 0 and #mesgs > options.limit then
+        mesgs = 'ALL'
+    else
+        mesgs = _make_range(mesgs)
+        mesgs = 'UID ' .. table.concat(mesgs, ',')
     end
 
-    if charset == nil then
-        if type(options.charset) == 'string' then
-            charset = options.charset
-        else
-            charset = ''
-        end
+    local query
+    if criteria == nil then
+        query = mesgs
+    elseif type(criteria) == 'string' then
+        query = mesgs .. ' ' .. criteria
+    else
+        query = _make_query(criteria, mesgs)
+    end
+
+    if type(options.charset) == 'string' then
+        charset = options.charset
+    else
+        charset = ''
     end
 
     if not self._check_connection(self) then return end
@@ -133,14 +146,15 @@ function Mailbox._flag_messages(self, mode, flags, messages)
 
     local f = ''
     if #flags ~= 0 then f = table.concat(flags, ' ') end
+
+    local r = false
     local m = _make_range(messages)
     local n = #m
-    local r = false
-    for i = 1, n, 50 do
-        j = i + 49
-        if n < j then
-            j = n
-        end
+    local l = n
+    if options.limit > 0 then l = options.limit end
+    for i = 1, n, l do
+        j = i + l - 1
+        if n < j then j = n end
         if not self._check_connection(self) then return end
         r = ifcore.store(self._account._account.session, table.concat(m, ',',
                          i, j), mode, f)
@@ -150,7 +164,7 @@ function Mailbox._flag_messages(self, mode, flags, messages)
 
     if options.close == true then self._cached_close(self) end
 
-    return true
+    return r
 end
 
 
@@ -163,11 +177,11 @@ function Mailbox._copy_messages(self, dest, messages)
 
         local m = _make_range(messages)
         local n = #m
-        for i = 1, n, 50 do
-            j = i + 49
-            if n < j then
-                j = n
-            end
+        local l = n
+        if options.limit > 0 then l = options.limit end
+        for i = 1, n, l do
+            j = i + l - 1
+            if n < j then j = n end
             if not self._check_connection(self) then return end
             r = ifcore.copy(self._account._account.session,
                             table.concat(m, ',', i, j), dest._mailbox)
@@ -196,7 +210,7 @@ function Mailbox._copy_messages(self, dest, messages)
         end
     end
 
-    return true
+    return r
 end
 
 
@@ -369,7 +383,6 @@ end
 
 function Mailbox._fetch_message(self, messages)
     if not messages or #messages == 0 then return end
-    if self._cached_select(self) ~= true then return end
 
     local header = self._fetch_header(self, messages)
     local body = self._fetch_body(self, messages)
@@ -385,8 +398,6 @@ function Mailbox._fetch_message(self, messages)
         end
     end
 
-    if options.close == true then self._cached_close(self) end
-
     return results
 end
 
@@ -488,10 +499,13 @@ end
 
 function Mailbox.check_status(self)
     if not self._check_connection(self) then return end
+    if self._account._account.selected == self._mailbox then
+        self._cached_close(self)
+    end
     local r, exist, recent, unseen, uidnext =
         ifcore.status(self._account._account.session,self._mailbox)
     self._check_result(self, 'status', r)
-    if r == false then return false end
+    if r == false then return -1, -1, -1, -1 end
 
     if options.info == true then
         print(exist .. ' messages, ' .. recent .. ' recent, ' .. unseen ..
@@ -502,8 +516,8 @@ function Mailbox.check_status(self)
 end
 
 
-function Mailbox.send_query(self, criteria, charset)
-    return Set(self._send_query(self, criteria, charset))
+function Mailbox.send_query(self, criteria, messages)
+    return Set(self._send_query(self, criteria, messages))
 end
 
 function Mailbox.select_all(self)
@@ -514,7 +528,7 @@ end
 function Mailbox.add_flags(self, flags, messages)
     _check_required(flags, 'table')
     _check_required(messages, 'table')
-    
+
     local mesgs = _extract_messages(self, messages)
     local r = self._flag_messages(self, 'add', flags, mesgs)
     if options.info == true and r == true then
@@ -785,166 +799,177 @@ function Mailbox.append_message(self, message, flags, date)
         print('Appended message of ' .. #message .. ' octets to ' ..
               self._string .. '.')
     end
-    
+
     return true
 end
 
 
-function Mailbox.is_answered(self)
-    return self.send_query(self, 'ANSWERED')
+function Mailbox.is_answered(self, messages)
+    return self.send_query(self, 'ANSWERED', messages)
 end
 
-function Mailbox.is_deleted(self)
-    return self.send_query(self, 'DELETED')
+function Mailbox.is_deleted(self, messages)
+    return self.send_query(self, 'DELETED', messages)
 end
 
-function Mailbox.is_draft(self)
-    return self.send_query(self, 'DRAFT')
+function Mailbox.is_draft(self, messages)
+    return self.send_query(self, 'DRAFT', messages)
 end
 
-function Mailbox.is_flagged(self)
-    return self.send_query(self, 'FLAGGED')
+function Mailbox.is_flagged(self, messages)
+    return self.send_query(self, 'FLAGGED', messages)
 end
 
-function Mailbox.is_new(self)
-    return self.send_query(self, 'NEW')
+function Mailbox.is_new(self, messages)
+    return self.send_query(self, 'NEW', messages)
 end
 
-function Mailbox.is_old(self)
-    return self.send_query(self, 'OLD')
+function Mailbox.is_old(self, messages)
+    return self.send_query(self, 'OLD', messages)
 end
 
-function Mailbox.is_recent(self)
-    return self.send_query(self, 'RECENT')
+function Mailbox.is_recent(self, messages)
+    return self.send_query(self, 'RECENT', messages)
 end
 
-function Mailbox.is_seen(self)
-    return self.send_query(self, 'SEEN')
+function Mailbox.is_seen(self, messages)
+    return self.send_query(self, 'SEEN', messages)
 end
 
-function Mailbox.is_unanswered(self)
-    return self.send_query(self, 'UNANSWERED')
+function Mailbox.is_unanswered(self, messages)
+    return self.send_query(self, 'UNANSWERED', messages)
 end
 
-function Mailbox.is_undeleted(self)
-    return self.send_query(self, 'UNDELETED')
+function Mailbox.is_undeleted(self, messages)
+    return self.send_query(self, 'UNDELETED', messages)
 end
 
-function Mailbox.is_undraft(self)
-    return self.send_query(self, 'UNDRAFT')
+function Mailbox.is_undraft(self, messages)
+    return self.send_query(self, 'UNDRAFT', messages)
 end
 
-function Mailbox.is_unflagged(self)
-    return self.send_query(self, 'UNFLAGGED')
+function Mailbox.is_unflagged(self, messages)
+    return self.send_query(self, 'UNFLAGGED', messages)
 end
 
-function Mailbox.is_unseen(self)
-    return self.send_query(self, 'UNSEEN')
-end
-
-function Mailbox.is_larger(self, size)
-    _check_required(size, 'number')
-    return self.send_query(self, 'LARGER ' .. tostring(size))
-end
-
-function Mailbox.is_smaller(self, size)
-    _check_required(size, 'number')
-    return self.send_query(self, 'SMALLER ' .. tostring(size))
+function Mailbox.is_unseen(self, messages)
+    return self.send_query(self, 'UNSEEN', messages)
 end
 
 
-function Mailbox.arrived_on(self, date)
-    _check_required(date, 'string')
-    return self.send_query(self, 'ON ' .. date)
-end
-
-function Mailbox.arrived_before(self, date)
-    _check_required(date, 'string')
-    return self.send_query(self, 'BEFORE ' .. date)
-end
-
-function Mailbox.arrived_since(self, date)
-    _check_required(date, 'string')
-    return self.send_query(self, 'SINCE ' .. date)
-end
-
-function Mailbox.sent_on(self, date)
-    _check_required(date, 'string')
-    return self.send_query(self, 'SENTON ' .. date)
-end
-
-function Mailbox.sent_before(self, date)
-    _check_required(date, 'string')
-    return self.send_query(self, 'SENTBEFORE ' .. date)
-end
-
-function Mailbox.sent_since(self, date)
-    _check_required(date, 'string')
-    return self.send_query(self, 'SENTSINCE ' .. date)
-end
-
-function Mailbox.is_newer(self, days)
-    _check_required(days, 'number')
-    return self.send_query(self, 'SINCE ' .. form_date(days))
-end
-
-function Mailbox.is_older(self, days)
-    _check_required(days, 'number')
-    return self.send_query(self, 'BEFORE ' .. form_date(days))
-end
-
-
-function Mailbox.has_flag(self, flag)
+function Mailbox.has_keyword(self, flag, message)
     _check_required(flag, 'string')
-    return self.send_query(self, 'KEYWORD "' .. flag .. '"')
+    return self.send_query(self, 'KEYWORD ' .. flag, messages)
+end
+
+Mailbox.has_flag = Mailbox.has_keyword
+
+function Mailbox.has_unkeyword(self, flag, message)
+    _check_required(flag, 'string')
+    return self.send_query(self, 'UNKEYWORD ' .. flag, messages)
 end
 
 
-function Mailbox.contain_field(self, field, string)
+function Mailbox.is_larger(self, size, messages)
+    _check_required(size, 'number')
+    _check_optional(messages, 'table')
+    return self.send_query(self, 'LARGER ' .. tostring(size), messages)
+end
+
+function Mailbox.is_smaller(self, size, messages)
+    _check_required(size, 'number')
+    _check_optional(messages, 'table')
+    return self.send_query(self, 'SMALLER ' .. tostring(size), messages)
+end
+
+
+function Mailbox.arrived_on(self, date, messages)
+    _check_required(date, 'string')
+    return self.send_query(self, 'ON ' .. date, messages)
+end
+
+function Mailbox.arrived_before(self, date, messages)
+    _check_required(date, 'string')
+    return self.send_query(self, 'BEFORE ' .. date, messages)
+end
+
+function Mailbox.arrived_since(self, date, messages)
+    _check_required(date, 'string')
+    return self.send_query(self, 'SINCE ' .. date, messages)
+end
+
+function Mailbox.sent_on(self, date, messages)
+    _check_required(date, 'string')
+    return self.send_query(self, 'SENTON ' .. date, messages)
+end
+
+function Mailbox.sent_before(self, date, messages)
+    _check_required(date, 'string')
+    return self.send_query(self, 'SENTBEFORE ' .. date, messages)
+end
+
+function Mailbox.sent_since(self, date, messages)
+    _check_required(date, 'string')
+    return self.send_query(self, 'SENTSINCE ' .. date, messages)
+end
+
+function Mailbox.is_newer(self, days, messages)
+    _check_required(days, 'number')
+    return self.send_query(self, 'SINCE ' .. form_date(days), messages)
+end
+
+function Mailbox.is_older(self, days, messages)
+    _check_required(days, 'number')
+    return self.send_query(self, 'BEFORE ' .. form_date(days), messages)
+end
+
+
+function Mailbox.contain_field(self, field, string, messages)
     _check_required(field, 'string')
     _check_required(string, 'string')
-    return self.send_query(self, 'HEADER ' .. field .. ' "' .. string .. '"')
+    return self.send_query(self, 'HEADER ' .. field .. ' "' .. string .. '"',
+                           messages)
 end
 
-function Mailbox.contain_bcc(self, string)
+function Mailbox.contain_bcc(self, string, messages)
     _check_required(string, 'string')
-    return self.send_query(self, 'BCC "' .. string .. '"')
+    return self.send_query(self, 'BCC "' .. string .. '"', messages)
 end
 
-function Mailbox.contain_cc(self, string)
+function Mailbox.contain_cc(self, string, messages)
     _check_required(string, 'string')
-    return self.send_query(self, 'CC "' .. string .. '"')
+    return self.send_query(self, 'CC "' .. string .. '"', messages)
 end
 
-function Mailbox.contain_from(self, string)
+function Mailbox.contain_from(self, string, messages)
     _check_required(string, 'string')
-    return self.send_query(self, 'FROM "' .. string .. '"')
+    return self.send_query(self, 'FROM "' .. string .. '"', messages)
 end
 
-function Mailbox.contain_subject(self, string)
+function Mailbox.contain_subject(self, string, messages)
     _check_required(string, 'string')
-    return self.send_query(self, 'SUBJECT "' .. string .. '"')
+    return self.send_query(self, 'SUBJECT "' .. string .. '"', messages)
 end
 
-function Mailbox.contain_to(self, string)
+function Mailbox.contain_to(self, string, messages)
     _check_required(string, 'string')
-    return self.send_query(self, 'TO "' .. string .. '"')
+    return self.send_query(self, 'TO "' .. string .. '"', messages)
 end
 
-function Mailbox.contain_header(self, string)
+function Mailbox.contain_header(self, string, messages)
     _check_required(string, 'string')
     return self.send_query(self, 'TEXT "' .. string .. '" NOT BODY "' ..
-        string .. '"')
+        string .. '"', messages)
 end
 
-function Mailbox.contain_body(self, string)
+function Mailbox.contain_body(self, string, messages)
     _check_required(string, 'string')
-    return self.send_query(self, 'BODY "' .. string .. '"')
+    return self.send_query(self, 'BODY "' .. string .. '"', messages)
 end
 
-function Mailbox.contain_message(self, string)
+function Mailbox.contain_message(self, string, messages)
     _check_required(string, 'string')
-    return self.send_query(self, 'TEXT "' .. string .. '"')
+    return self.send_query(self, 'TEXT "' .. string .. '"', messages)
 end
 
 
@@ -1039,15 +1064,19 @@ end
 
 function Mailbox.enter_idle(self)
     if self._cached_select(self) ~= true then return false end
-   
+
     if not self._check_connection(self) then return end
-    local r = ifcore.idle(self._account._account.session)
+    local r, event = ifcore.idle(self._account._account.session)
     self._check_result(self, 'idle', r)
     if r == false then return false end
 
     if options.close == true then self._cached_close(self) end
 
-    return true
+    if type(event) == 'string' then
+        return true, string.upper(event)
+    else
+        return true
+    end
 end
 
 
