@@ -33,10 +33,16 @@ int send_continuation(session *ssn, const char *data, size_t len);
 	switch ((F)) {							       \
 	case -1:							       \
 		if ((!strcasecmp(get_option_string("recover"), "all") ||       \
-		    !strcasecmp(get_option_string("recover"), "errors")) &&    \
-		    request_login(&ssn, NULL, NULL, NULL, NULL, NULL,	       \
-		    	NULL) != -1)   					       \
-			return STATUS_NONE;				       \
+		    !strcasecmp(get_option_string("recover"), "errors")))      \
+			for (;;) {					       \
+				if (request_login(&ssn, NULL, NULL, NULL,      \
+				    NULL, NULL, NULL) != -1)		       \
+					return STATUS_NONE;		       \
+				if (get_option_boolean("persist"))	       \
+					sleep(WAIT_RETRY_TIMEOUT);	       \
+				else					       \
+					break;				       \
+			}						       \
 		return -1;						       \
 	case STATUS_BYE:						       \
 		close_connection(ssn);					       \
@@ -156,7 +162,7 @@ int
 request_login(session **ssnptr, const char *server, const char *port, const
     char *ssl, const char *user, const char *pass, const char *oauth2)
 {
-	int t, r, rg = -1, rl = -1; 
+	int t, r, rg = -1, rl = -1;
 	session *ssn = *ssnptr;
 	
 	if (*ssnptr && (*ssnptr)->socket != -1)
@@ -609,7 +615,7 @@ request_store(session *ssn, const char *mesg, const char *mode, const char
 	if (opts.dryrun)
 		return STATUS_DRYRUN;
 
-	TRY(t = send_request(ssn, "UID STORE %s %sFLAGS.SILENT (%s)", mesg, 
+	TRY(t = send_request(ssn, "UID STORE %s %sFLAGS.SILENT (%s)", mesg,
 	    (!strncasecmp(mode, "add", 3) ? "+" :
 	    !strncasecmp(mode, "remove", 6) ? "-" : ""), flags));
 	TRY(r = response_generic(ssn, t));
@@ -675,7 +681,7 @@ request_append(session *ssn, const char *mbox, const char *mesg, size_t
 	    (date ? date : ""), (date ? "\"" : ""), mesglen));
 	TRY(r = response_continuation(ssn, t));
 	if (r == STATUS_CONTINUE) {
-		TRY(send_continuation(ssn, mesg, mesglen)); 
+		TRY(send_continuation(ssn, mesg, mesglen));
 		TRY(r = response_generic(ssn, t));
 	}
 
@@ -692,7 +698,7 @@ request_append(session *ssn, const char *mbox, const char *mesg, size_t
 		    (date ? date : ""), (date ? "\"" : ""), mesglen));
 		TRY(r = response_continuation(ssn, t));
 		if (r == STATUS_CONTINUE) {
-			TRY(send_continuation(ssn, mesg, mesglen)); 
+			TRY(send_continuation(ssn, mesg, mesglen));
 			TRY(r = response_generic(ssn, t));
 		}
 	}
